package jp.veltec.pdf;

import java.util.HashMap;
import java.io.OutputStream;
import java.io.IOException;
import java.io.File;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.Font;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Paragraph;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Element;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.BaseColor;
import com.itextpdf.text.Image;
import com.itextpdf.text.pdf.ColumnText;
import com.itextpdf.text.pdf.PdfPageEventHelper;

import com.itextpdf.text.BadElementException;
import java.net.MalformedURLException;

/**
 *複数ページにわたる表を含む帳票を作成します。
 */
public class ListTableBuilder extends PdfPageEventHelper{

	OutputStream fOutputStream;

	org.w3c.dom.Element fTemplate;
    Document fDocument;
	float fDocumentHeight;

	PdfWriter fWriter;
	HashMap<String,String> fTableDefault;
	HashMap<String,String> fRowDefault;
	HashMap<String,String> fData;

	NodeList fHeaderTrs;
	NodeList fRecordTrs;
	NodeList fFooterTrs;

	FontManager fFontManager;
	PdfPTable fPdfTable;
	int fRows;
	int fRowCounter;
	int fCols;
	int fHeaderRows;
	boolean[][] fMatrix;
	boolean fRepeat;
	String fName;

	PageNumberWriter fPageNumberWriter;

	public ListTableBuilder(OutputStream aOutputStream){
		fOutputStream = aOutputStream;
		try{
			fDocument = new Document();
			fFontManager = new FontManager();
	        fWriter = PdfWriter.getInstance(fDocument, fOutputStream);
		}catch(DocumentException e){
			e.printStackTrace();
			throw new IllegalStateException(e.getMessage());
		}
	}

	FontManager getFontManager(){
		return fFontManager;
	}

	public void setDefaultFontPath(String aFontPath){
		fFontManager.setDefaultFontPath(aFontPath);
	}

	public void setTemplateFile(File aFile){
		try{
			DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
			org.w3c.dom.Document xmlDocument = builder.parse(aFile);
			org.w3c.dom.Element xmlElement = xmlDocument.getDocumentElement();
			setTemplate(xmlElement);
		}catch(org.xml.sax.SAXException e){
			e.printStackTrace();
			throw new IllegalStateException(e.getMessage());
		}catch(javax.xml.parsers.ParserConfigurationException e){
			e.printStackTrace();
			throw new IllegalStateException(e.getMessage());
		}catch(IOException e){
			e.printStackTrace();
			throw new IllegalStateException(e.getMessage());
		}
	}

	public void setTemplate(org.w3c.dom.Element aTemplate){
		fTemplate=aTemplate;
		Rectangle mSize = PageSize.A4;
		String mAttribute = fTemplate.getAttribute("size");
		if(!mAttribute.equals("")){
			if(mAttribute.equals("A4P")||mAttribute.equals("A4")){
				//default;
			}else if(mAttribute.equals("A4L")){
					float x = getSize("297mm");
					float y = getSize("210mm");
					mSize = new Rectangle(x,y);
			}else if(mAttribute.equals("A5")){
					float x = getSize("148.5mm");
					float y = getSize("210mm");
					mSize = new Rectangle(x,y);
			}else if(mAttribute.equals("A5L")){
					float x = getSize("210mm");
					float y = getSize("148.5mm");
					mSize = new Rectangle(x,y);
			}else{
				try{
					String[] size = mAttribute.split(" ");
					float x = getSize(size[0]);
					float y = getSize(size[1]);
					mSize = new Rectangle(x,y);
				}catch(Exception e){
					throw new IllegalArgumentException("page size not supported:" + mAttribute);
				}
			}
		}
		fDocument.setPageSize(mSize);

		float mMarginLeft=getSize("15mm");
		mAttribute  = fTemplate.getAttribute("margin-left");
		if(!mAttribute.equals("")){
			mMarginLeft=getSize(mAttribute);
		}

		float mMarginRight=getSize("15mm");
		mAttribute  = fTemplate.getAttribute("margin-right");
		if(!mAttribute.equals("")){
			mMarginRight=getSize(mAttribute);
		}

		float mMarginTop=getSize("15mm");
		mAttribute  = fTemplate.getAttribute("margin-top");
		if(!mAttribute.equals("")){
			mMarginTop=getSize(mAttribute);
		}

		float mMarginBottom=getSize("15mm");
		mAttribute  = fTemplate.getAttribute("margin-bottom");
		if(!mAttribute.equals("")){
			mMarginBottom=getSize(mAttribute);
		}

		NodeList fonts = fTemplate.getElementsByTagName("font");
		if(fonts.getLength()!=0){
			fFontManager.register(fonts);
		}

		NodeList pageNumbers = fTemplate.getElementsByTagName("pagenumber");
		if(pageNumbers.getLength()!=0){
			setPageLabel((org.w3c.dom.Element)pageNumbers.item(0));
		}

		fDocument.setMargins(mMarginLeft,mMarginRight,mMarginTop,mMarginBottom);
        fDocument.open();
		beginTable();
	}

	public void setPageLabel(org.w3c.dom.Element pageLabel){
		float mMargin = getSize("5mm");
		String mAttribute = pageLabel.getAttribute("margin");
		if(!mAttribute.equals("")){
			mMargin = getSize(mAttribute);
		}
		float mPositionX=0;
		float mPositionY=0;
		int mAlign=Element.ALIGN_LEFT;
		mAttribute = pageLabel.getAttribute("position");
		if(mAttribute.equals("top-left")){
			mPositionX=mMargin;
			mPositionY=fDocument.getPageSize().getHeight()-mMargin;
		}else if(mAttribute.equals("top-center")){
			mPositionX=fDocument.getPageSize().getWidth()/2;
			mPositionY=fDocument.getPageSize().getHeight()-mMargin;
			mAlign=Element.ALIGN_CENTER;
		}else if(mAttribute.equals("top-right")){
			mPositionX=fDocument.getPageSize().getWidth()-mMargin;
			mPositionY=fDocument.getPageSize().getHeight()-mMargin;
			mAlign=Element.ALIGN_CENTER;
		}else if(mAttribute.equals("bottom-left")){
			mPositionX=mMargin;
			mPositionY=mMargin;
		}else if(mAttribute.equals("bottom-center")||mAttribute.equals("")){
			mPositionX=fDocument.getPageSize().getWidth()/2;
			mPositionY=mMargin;
			mAlign=Element.ALIGN_CENTER;
		}else if(mAttribute.equals("bottom-right")){
			mPositionX=fDocument.getPageSize().getWidth()-mMargin;
			mPositionY=mMargin;
			mAlign=Element.ALIGN_CENTER;
		}else{
			mPositionX = getSize(mAttribute.split(" ")[0]);
			mPositionY = getSize(mAttribute.split(" ")[1]);
		}
		
		fPageNumberWriter = new PageNumberWriter(this);
		fPageNumberWriter.setPosition(mPositionX,mPositionY);
		fPageNumberWriter.setAlign(mAlign);
		mAttribute = pageLabel.getAttribute("text-before");
		if(!mAttribute.equals("")){
			fPageNumberWriter.setTextBefore(mAttribute);
		}
		mAttribute = pageLabel.getAttribute("text-after");
		if(!mAttribute.equals("")){
			fPageNumberWriter.setTextAfter(mAttribute);
		}
		mAttribute = pageLabel.getAttribute("font-size");
		if(!mAttribute.equals("")){
			fPageNumberWriter.setFontSize(getSize(mAttribute));
		}
		mAttribute = pageLabel.getAttribute("font-family");
		if(!mAttribute.equals("")){
			fPageNumberWriter.setFontFamily(mAttribute);
		}
		mAttribute = pageLabel.getAttribute("font-style");
			//NOP
		if(mAttribute.equals("")){
		}else if(mAttribute.equals("bold")){
			fPageNumberWriter.setFontStyle(1);
		}else if(mAttribute.equals("italic")){
			fPageNumberWriter.setFontStyle(2);
		}else if(mAttribute.equals("bold-italic")){
			fPageNumberWriter.setFontStyle(3);
		}else{
			throw new IllegalArgumentException("not supported font-style:" + mAttribute);
		}
		fWriter.setPageEvent(fPageNumberWriter);
	}

	public void setPageNumberFontFamily(String aFontFamily){
		fPageNumberWriter.setFontFamily(aFontFamily);
	}

	public void setPageNumber(int aPageNumber){
		fPageNumberWriter.setPageNumber(aPageNumber);
	}

    private void beginTable() throws IllegalStateException{
		try{
			NodeList tables = fTemplate.getElementsByTagName("table");
			org.w3c.dom.Element xmlTable = (org.w3c.dom.Element)tables.item(0);
			String strCols = xmlTable.getAttribute("cols");
			org.w3c.dom.Element header = (org.w3c.dom.Element)xmlTable.getElementsByTagName("header").item(0);
			if(header!=null){
				fHeaderTrs = header.getElementsByTagName("tr");
				fHeaderRows = fHeaderTrs.getLength();
			}
			org.w3c.dom.Element record = (org.w3c.dom.Element)xmlTable.getElementsByTagName("record").item(0);
			if(record!=null){
				fRecordTrs = record.getElementsByTagName("tr");
			}
			org.w3c.dom.Element footer = (org.w3c.dom.Element)xmlTable.getElementsByTagName("footer").item(0);
			if(footer!=null){
				fFooterTrs = footer.getElementsByTagName("tr");
			}
			if(strCols.equals("")){
				throw new IllegalStateException("cols not specified");
			}
			fCols = Integer.parseInt(strCols);
			fPdfTable = new PdfPTable(fCols);
			fPdfTable.setLockedWidth(true);
			fPdfTable.setTotalWidth(fDocument.getPageSize().getWidth());
			fTableDefault = new HashMap<String,String>();
			NamedNodeMap mapT = xmlTable.getAttributes();
			fRepeat=false;
			for(int i=0;i<mapT.getLength();i++){
				Node node = mapT.item(i);
				if(node.getNodeName().equals("widths")){
					String[] strWidths = node.getNodeValue().split(" ");
					float[] widths = new float[strWidths.length];
					float totalWidth=0;
					for(int j=0;j<widths.length;j++){
						widths[j]=getSize(strWidths[j]);
						totalWidth+=widths[j];
					}
					fPdfTable.setWidths(widths);
					fPdfTable.setTotalWidth(totalWidth);
				}else if(node.getNodeName().equals("name")){
					fName = node.getNodeValue();
				}else{
					fTableDefault.put(node.getNodeName(),node.getNodeValue());
				}
			}
		}catch(DocumentException e){
			throw new IllegalStateException(e.getMessage());
		}
    }

	
	public void addHeader() throws DocumentException{
		addHeader(new HashMap<String,String>());
	}
	public void addHeader(HashMap<String,String> aData) throws DocumentException{
		fPdfTable.setHeaderRows(fHeaderRows);
		if(fHeaderTrs==null){
			throw new IllegalStateException("header is not found in the template");
		}
		fData = aData;
		for(int i=0;i<fHeaderTrs.getLength();i++){
			org.w3c.dom.Element tr = (org.w3c.dom.Element)fHeaderTrs.item(i);
			fRowDefault = new HashMap<String,String>();
			NamedNodeMap mapR = tr.getAttributes();
			for(int j=0;j<mapR.getLength();j++){
				Node nodeR = mapR.item(j);
				fRowDefault.put(nodeR.getNodeName(),nodeR.getNodeValue());
			}
			processRow(tr);
		}
	}

	public void addRecord() throws DocumentException{
		addRecord(new HashMap<String,String>());
	}
	public void addRecord(HashMap<String,String> aData) throws DocumentException{
		if(fRecordTrs==null){
			throw new IllegalStateException("record is not found in the template");
		}
		fData = aData;
		for(int i=0;i<fRecordTrs.getLength();i++){
			org.w3c.dom.Element tr = (org.w3c.dom.Element)fRecordTrs.item(i);
			fRowDefault = new HashMap<String,String>();
			NamedNodeMap mapR = tr.getAttributes();
			for(int j=0;j<mapR.getLength();j++){
				Node nodeR = mapR.item(j);
				fRowDefault.put(nodeR.getNodeName(),nodeR.getNodeValue());
			}
			processRow(tr);
		}
	}

	public void addFooter() throws DocumentException{
		addFooter(new HashMap<String,String>());
	}
	public void addFooter(HashMap<String,String> aData) throws DocumentException{
		if(fFooterTrs==null){
			throw new IllegalStateException("footer is not found in the template");
		}
		fData = aData;
		for(int i=0;i<fFooterTrs.getLength();i++){
			org.w3c.dom.Element tr = (org.w3c.dom.Element)fFooterTrs.item(i);
			fRowDefault = new HashMap<String,String>();
			NamedNodeMap mapR = tr.getAttributes();
			for(int j=0;j<mapR.getLength();j++){
				Node nodeR = mapR.item(j);
				fRowDefault.put(nodeR.getNodeName(),nodeR.getNodeValue());
			}
			processRow(tr);
		}
	}
	

	public void newPage() throws DocumentException{
		fDocument.add(fPdfTable);
		fDocument.newPage();
		beginTable();
	}

	public void close() throws DocumentException{
		fDocument.add(fPdfTable);
        fDocument.close();
	}

	void processRow(org.w3c.dom.Element fTr) throws DocumentException{
		NodeList tds = fTr.getElementsByTagName("td");
		boolean isLeftEnd;
		boolean isRightEnd;
		boolean isBottomEnd;
		for(int k=0;k<tds.getLength();k++){
			org.w3c.dom.Element td = (org.w3c.dom.Element)tds.item(k);
			String content = td.getTextContent();
			String id = td.getAttribute("id");
			if(!id.equals("") && fData.get(id)!=null){
				content = fData.get(id);
			}
			Cell cell = new Cell();
			String strColspan = td.getAttribute("colspan");
			int colspan = 1;
			if(!strColspan.equals("")){
				colspan = Integer.parseInt(strColspan);
				cell.setColspan(colspan);
			}
			int colCounter=0;
			int rowspan = 1;
			String strRowspan = td.getAttribute("rowspan");
			if(!strRowspan.equals("")){
				rowspan = Integer.parseInt(strRowspan);
				cell.setRowspan(rowspan);
			}
			cell.setAttributes(td);
			cell.setContent(content);
			fPdfTable.addCell(cell);
		}
	}

	class Cell extends PdfPCell{
		HashMap<String,String> attributes = new HashMap<String,String>();
		BaseFont bf = fFontManager.getBaseFont("明朝");
		Font fFont  = new Font(bf, 12);
		float fFontSize = 12f;
		float fLineHeight=1f;
		float fCharacterSpacing=0f;
		void setAttributes(org.w3c.dom.Element td){
			String attribute;
			attributes.putAll(fTableDefault);
			attributes.putAll(fRowDefault);
			NamedNodeMap map = td.getAttributes();
			for(int i=0;i<map.getLength();i++){
				Node node = map.item(i);
				attributes.put(node.getNodeName(),node.getNodeValue());
			}
			attribute = attributes.get("min-height");
			if(attribute!=null){
				setMinimumHeight(getSize(attribute));
			}
			attribute = attributes.get("height");
			if(attribute!=null){
				setFixedHeight(getSize(attribute));
			}
			attribute = attributes.get("align-h");
			if(attribute!=null){
				if(attribute.equals("center")){
					setHorizontalAlignment(Element.ALIGN_CENTER);
				}else if(attribute.equals("right")){
					setHorizontalAlignment(Element.ALIGN_RIGHT);
				}else if(attribute.equals("left")){
					setHorizontalAlignment(Element.ALIGN_LEFT);
				}
			}
			attribute = attributes.get("align-v");
			if(attribute!=null){
				if(attribute.equals("middle")){
					setVerticalAlignment(Element.ALIGN_MIDDLE);
				}else if(attribute.equals("bottom")){
					setVerticalAlignment(Element.ALIGN_BOTTOM);
				}else if(attribute.equals("top")){
					setVerticalAlignment(Element.ALIGN_TOP);
				}
			}
			attribute = attributes.get("font-family");
			if(attribute!=null){
				BaseFont bf = fFontManager.getBaseFont(attribute);
				fFont = new Font(bf,fFontSize);
			}
			attribute = attributes.get("font-style");
			if(attribute!=null){
				fFont.setStyle(attribute);
			}
			attribute = attributes.get("font-size");
			if(attribute!=null){
				fFontSize=getSize(attribute);
				fFont.setSize(fFontSize);
			}
			attribute=attributes.get("font-color");
			if(attribute!=null){
				fFont.setColor(getColor(attribute));
			}
			attribute=attributes.get("background-color");
			if(attribute!=null){
				setBackgroundColor(getColor(attribute));
			}
			attribute = attributes.get("border-width");
			if(attribute!=null){
				//setBorderWidth(getSize(attribute));
				setBorderWidthTop(getSize(attribute));
				setBorderWidthBottom(getSize(attribute));
				setBorderWidthLeft(getSize(attribute));
				setBorderWidthRight(getSize(attribute));
			}
			attribute = attributes.get("border-width-top");
			if(attribute!=null){
				setBorderWidthTop(getSize(attribute));
			}
			attribute = attributes.get("border-width-bottom");
			if(attribute!=null){
				setBorderWidthBottom(getSize(attribute));
			}
			attribute = attributes.get("border-width-left");
			if(attribute!=null){
				setBorderWidthLeft(getSize(attribute));
			}
			attribute = attributes.get("border-width-right");
			if(attribute!=null){
				setBorderWidthRight(getSize(attribute));
			}
			attribute = attributes.get("border-color");
			if(attribute!=null){
				setBorderColor(getColor(attribute));
			}
			attribute = attributes.get("border-color-left");
			if(attribute!=null){
				setBorderColorLeft(getColor(attribute));
			}
			attribute = attributes.get("border-color-right");
			if(attribute!=null){
				setBorderColorRight(getColor(attribute));
			}
			attribute = attributes.get("border-color-top");
			if(attribute!=null){
				setBorderColorTop(getColor(attribute));
			}
			attribute = attributes.get("border-color-bottom");
			if(attribute!=null){
				setBorderColorBottom(getColor(attribute));
			}
			attribute = attributes.get("padding");
			if(attribute!=null){
				setPadding(getSize(attribute));
			}
			attribute = attributes.get("padding-left");
			if(attribute!=null){
				setPaddingLeft(getSize(attribute));
			}
			attribute = attributes.get("padding-right");
			if(attribute!=null){
				setPaddingRight(getSize(attribute));
			}
			attribute = attributes.get("padding-top");
			if(attribute!=null){
				setPaddingTop(getSize(attribute));
			}
			attribute = attributes.get("padding-bottom");
			if(attribute!=null){
				setPaddingBottom(getSize(attribute));
			}
			attribute = attributes.get("rotation");
			if(attribute!=null){
				if(attribute.equals("left")){
					setRotation(90);
				}else if(attribute.equals("right")){
					setRotation(270);
				}else if(attribute.equals("up")){
					setRotation(0);
				}else if(attribute.equals("down")){
					setRotation(180);
				}
			}
			attribute = attributes.get("line-height");
			if(attribute!=null){
				setLeading(0,Float.parseFloat(attribute));
			}
			attribute = attributes.get("character-spacing");
			if(attribute!=null){
				fCharacterSpacing=getSize(attribute);
			}
		}

		void setContent(String content){
			content=content.replaceAll("\\\\n","\n");
			Paragraph p = new Paragraph(content,fFont);
			setPhrase(p);
		}
	}

	float getSize(String size) throws IllegalArgumentException{
		if(size.endsWith("mm")){
			size=size.substring(0,size.length()-2);
			return 72f/25.4f*(Float.parseFloat(size));
		}else if(size.endsWith("pt")){
			size=size.substring(0,size.length()-2);
			return Float.parseFloat(size);
		}else{
			throw new IllegalArgumentException("size format error:"+ size);
		}
	}

	BaseColor getColor(String color){
		if(color.equals("white")){
			color="#FFFFFF";
		}else if(color.equals("black")){
			color="#000000";
		}else if(color.equals("gray")){
			color="#808080";
		}else if(color.equals("red")){
			color="#FF0000";
		}else if(color.equals("green")){
			color="#00FF00";
		}else if(color.equals("blue")){
			color="#0000FF";
		}else if(color.equals("yellow")){
			color="#FFFF00";
		}
		try{
			int r =Integer.valueOf(color.substring(1,3),16);
			int g =Integer.valueOf(color.substring(3,5),16);
			int b =Integer.valueOf(color.substring(5,7),16);
			return new BaseColor(r,g,b);
		}catch(NumberFormatException e){
			throw new IllegalArgumentException("not supported color:" + color);
		}
	}

}
