package jp.veltec.pdf;
import java.io.IOException;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.Element;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.pdf.PdfWriter;
import com.itextpdf.text.pdf.PdfPageEventHelper;
import com.itextpdf.text.pdf.ColumnText;
import com.itextpdf.text.PageSize;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.Font;
import com.itextpdf.text.pdf.BaseFont;

class PageNumberWriter extends PdfPageEventHelper{
	int fPageNumber=1;
	float fPositionX = 0f;
	float fPositionY = 0f;
	int fAlign = Element.ALIGN_LEFT;
	String fTextBefore = "";
	String fTextAfter = "";
	Rectangle rect = PageSize.A4;
	BaseFont fBaseFont;
	FontManager fFontManager;
	Font fFont;
	float fFontSize = 12f;
	int fFontStyle;

	public PageNumberWriter(ListTableBuilder builder){
		fFontManager = builder.getFontManager();
		fBaseFont = fFontManager.getBaseFont("default");
		fFont = new Font(fBaseFont, fFontSize);
	}

	public void setPageNumber(int aPageNumber){
		fPageNumber = aPageNumber;
	}

	public void setFontFamily(String aFontFamily){
		fBaseFont = fFontManager.getBaseFont(aFontFamily);
		fFont = new Font(fBaseFont,fFontSize);
		fFont.setStyle(fFontStyle);
	}

	public void setFontSize(float aFontSize){
		fFontSize = aFontSize;
		fFont.setSize(aFontSize);
	}

	public void setFontStyle(int aFontStyle){
		fFontStyle = aFontStyle;
		fFont.setStyle(fFontStyle);
	}

	public void setPosition(float aPositionX,float aPositionY){
		fPositionX = aPositionX;
		fPositionY = aPositionY;
	}

	public void setAlign(int aAlign){
		fAlign = aAlign;
	}

	public void setTextBefore(String aText){
		fTextBefore = aText;
	}

	public void setTextAfter(String aText){
		fTextAfter = aText;
	}

	public void onStartPage(PdfWriter writer,Document document){
	}

	public void onEndPage(PdfWriter writer,Document document){
		String mText = fTextBefore + Integer.toString(fPageNumber) + fTextAfter;
        ColumnText.showTextAligned(
			writer.getDirectContent(),
            fAlign, 
			new Phrase(mText,fFont),
            fPositionX, 
			fPositionY,
			0
		);
		fPageNumber++;
	}

}
