package jp.veltec.pdf;

import java.util.HashMap;
import java.io.File;
import java.io.OutputStream;
import java.io.IOException;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.Font;
import com.itextpdf.text.Phrase;
import com.itextpdf.text.Chunk;
import com.itextpdf.text.Element;

import java.net.MalformedURLException;

/**
*1ページに複数の表を含む帳票を作成します。
*/
public class TableBuilder extends PdfBuilder{

	HashMap<String,String> fTableDefault;
	HashMap<String,String> fRowDefault;

	public TableBuilder(OutputStream aOutputStream){
		super(aOutputStream);
	}
    public void addPage(HashMap<String,String> aData) throws IllegalStateException{
		try{
			processPDF();
			processLine();
			processRectangle();
			processImage(true);
			processTable(aData);
			processImage(false);
			fDocument.newPage();
		}catch(MalformedURLException e){
			throw new IllegalStateException(e.getMessage());
		}catch(IOException e){
			throw new IllegalStateException(e.getMessage());
		}catch(DocumentException e){
			throw new IllegalStateException(e.getMessage());
		}
    }
	void processTable(HashMap<String,String> aData) throws DocumentException{
		NodeList tables = fTemplate.getElementsByTagName("table");
		for(int tableCount=0;tableCount<tables.getLength();tableCount++){
			org.w3c.dom.Element xmlTable = (org.w3c.dom.Element)tables.item(tableCount);
			String strCols = xmlTable.getAttribute("cols");
			if(strCols.equals("")){
				throw new IllegalStateException("cols not specified");
			}
			int cols = Integer.parseInt(strCols);
			PdfPTable pdfTable = new PdfPTable(cols);

			float outerBorderWidth = getSize("0.2mm");
			float innerBorderWidth = getSize("0.2mm");
			String attribute;
			attribute=xmlTable.getAttribute("outer-border-width");
			if(!attribute.equals("")){
				outerBorderWidth=getSize(attribute);
			}
			attribute=xmlTable.getAttribute("inner-border-width");
			if(!attribute.equals("")){
				innerBorderWidth=getSize(attribute);
			}
			attribute=xmlTable.getAttribute("margin-top");
			if(!attribute.equals("")){
				pdfTable.setSpacingBefore(getSize(attribute));
			}
			attribute=xmlTable.getAttribute("margin-bottom");
			if(!attribute.equals("")){
				pdfTable.setSpacingAfter(getSize(attribute));
			}
			attribute=xmlTable.getAttribute("table-align");
			if(!attribute.equals("")){
				if(attribute.equals("left")){
					pdfTable.setHorizontalAlignment(0);
				}else if(attribute.equals("center")){
					pdfTable.setHorizontalAlignment(1);
				}else if(attribute.equals("right")){
					pdfTable.setHorizontalAlignment(2);
				}else{
					throw new IllegalArgumentException("bad table-align:" + attribute);
				}
			}
			pdfTable.setLockedWidth(true);
			float totalWidth=0;
			attribute=xmlTable.getAttribute("widths");
			if(!attribute.equals("")){
				String[] strWidths = attribute.split(" ");
				float[] widths = new float[strWidths.length];
				for(int j=0;j<strWidths.length;j++){
					widths[j]=getSize(strWidths[j]);
					totalWidth += widths[j];
				}
				pdfTable.setWidths(widths);
			}
			if(totalWidth==0){
				pdfTable.setTotalWidth(fDocument.getPageSize().getWidth()-fDocument.leftMargin()-fDocument.rightMargin());
			}else{
				pdfTable.setTotalWidth(totalWidth);
			}
			fTableDefault = new HashMap<String,String>();
			NamedNodeMap mapT = xmlTable.getAttributes();
			for(int i=0;i<mapT.getLength();i++){
				Node node = mapT.item(i);
				fTableDefault.put(node.getNodeName(),node.getNodeValue());
			}

			NodeList trs = xmlTable.getElementsByTagName("tr");
			int rows=trs.getLength();
			boolean[][] matrix = new boolean[rows][cols];
			boolean isTopEnd=true;
			for(int rowCounter=0;rowCounter<rows;rowCounter++){
				org.w3c.dom.Element tr = (org.w3c.dom.Element)trs.item(rowCounter);
				fRowDefault = new HashMap<String,String>();
				NamedNodeMap mapR = tr.getAttributes();
				for(int i=0;i<mapR.getLength();i++){
					Node nodeR = mapR.item(i);
					fRowDefault.put(nodeR.getNodeName(),nodeR.getNodeValue());
				}
				NodeList tds = tr.getElementsByTagName("td");
				boolean isLeftEnd;
				boolean isRightEnd;
				boolean isBottomEnd;
				for(int k=0;k<tds.getLength();k++){
					org.w3c.dom.Element td = (org.w3c.dom.Element)tds.item(k);
					String content = td.getTextContent();
					String id = td.getAttribute("id");
					if(!id.equals("") && aData.get(id)!=null){
						content = aData.get(id);
					}

					Cell cell = new Cell();
					String strColspan = td.getAttribute("colspan");
					int colspan = 1;
					if(!strColspan.equals("")){
						colspan = Integer.parseInt(strColspan);
						cell.setColspan(colspan);
					}
					int colCounter=0;
					for(colCounter=0;colCounter<cols;colCounter++){
						if(matrix[rowCounter][colCounter]==false){
							break;
						}
					}
					if(colCounter==0){
						isLeftEnd=true;
					}else{
						isLeftEnd=false;
					}
					if(colCounter+colspan==cols){
						isRightEnd=true;
					}else{
						isRightEnd=false;
					}
					int rowspan = 1;
					String strRowspan = td.getAttribute("rowspan");
					if(!strRowspan.equals("")){
						rowspan = Integer.parseInt(strRowspan);
						cell.setRowspan(rowspan);
					}
					for(int r=0;r<rowspan;r++){
						for(int c=0;c<colspan;c++){
							matrix[rowCounter+r][colCounter+c]=true;
						}
					}
					if(rowCounter+rowspan==rows){
						isBottomEnd=true;
					}else{
						isBottomEnd=false;
					}
					cell.setBorderWidthLeft(innerBorderWidth);
					cell.setBorderWidthRight(innerBorderWidth);
					cell.setBorderWidthTop(innerBorderWidth);
					cell.setBorderWidthBottom(innerBorderWidth);
					if(isTopEnd){
						cell.setBorderWidthTop(outerBorderWidth);
					}
					if(isLeftEnd){
						cell.setBorderWidthLeft(outerBorderWidth);
					}
					if(isRightEnd){
						cell.setBorderWidthRight(outerBorderWidth);
					}
					if(isBottomEnd){
						cell.setBorderWidthBottom(outerBorderWidth);
					}

					cell.setAttributes(td);
					cell.setContent(content);
					pdfTable.addCell(cell);
				}
				
				isTopEnd=false;
			}
			attribute=xmlTable.getAttribute("position");
			if(attribute.equals("absolute")){
				if(xmlTable.getAttribute("left").equals("") ||xmlTable.getAttribute("top").equals("")){
					throw new IllegalArgumentException("left and top must be set for absolute position table");
				}
				float x=(getSize(xmlTable.getAttribute("left")));
				float y=(fDocument.getPageSize().getHeight()-getSize(xmlTable.getAttribute("top")));
				pdfTable.writeSelectedRows(0,-1,x,y,fDirectContent);
			}else{
				fDocument.add(pdfTable);
			}
		}
	}

	class Cell extends PdfPCell{
		HashMap<String,String> attributes = new HashMap<String,String>();
		Font fFont  = new Font(fFontManager.getBaseFont("明朝"), 12);
		float fFontSize = 12f;
		float fLineHeight=1f;
		float fCharacterSpacing=0f;
		void setAttributes(org.w3c.dom.Element td){
			String attribute;
			attributes.putAll(fTableDefault);
			attributes.putAll(fRowDefault);
			NamedNodeMap map = td.getAttributes();
			for(int i=0;i<map.getLength();i++){
				Node node = map.item(i);
				attributes.put(node.getNodeName(),node.getNodeValue());
			}
			attribute = attributes.get("min-height");
			if(attribute!=null){
				setMinimumHeight(getSize(attribute));
			}
			attribute = attributes.get("height");
			if(attribute!=null){
				setFixedHeight(getSize(attribute));
			}
			attribute = attributes.get("align-h");
			if(attribute!=null){
				if(attribute.equals("center")){
					setHorizontalAlignment(Element.ALIGN_CENTER);
				}else if(attribute.equals("right")){
					setHorizontalAlignment(Element.ALIGN_RIGHT);
				}else if(attribute.equals("left")){
					setHorizontalAlignment(Element.ALIGN_LEFT);
				}
			}
			attribute = attributes.get("align-v");
			if(attribute!=null){
				if(attribute.equals("middle")){
					setVerticalAlignment(Element.ALIGN_MIDDLE);
				}else if(attribute.equals("bottom")){
					setVerticalAlignment(Element.ALIGN_BOTTOM);
				}else if(attribute.equals("top")){
					setVerticalAlignment(Element.ALIGN_TOP);
				}
			}
			attribute = attributes.get("font-family");
			if(attribute!=null){
				fFont = new Font(fFontManager.getBaseFont(attribute),fFontSize);
			}
			attribute = attributes.get("font-style");
			if(attribute!=null){
				fFont.setStyle(attribute);
			}
			attribute = attributes.get("font-size");
			if(attribute!=null){
				fFontSize=getSize(attribute);
				fFont.setSize(fFontSize);
			}
			attribute=attributes.get("font-color");
			if(attribute!=null){
				fFont.setColor(getColor(attribute));
			}
			attribute=attributes.get("background-color");
			if(attribute!=null){
				setBackgroundColor(getColor(attribute));
			}
			attribute = attributes.get("border-width");
			if(attribute!=null){
				//setBorderWidth(getSize(attribute));
				setBorderWidthTop(getSize(attribute));
				setBorderWidthBottom(getSize(attribute));
				setBorderWidthLeft(getSize(attribute));
				setBorderWidthRight(getSize(attribute));
			}
			attribute = attributes.get("border-width-top");
			if(attribute!=null){
				setBorderWidthTop(getSize(attribute));
			}
			attribute = attributes.get("border-width-bottom");
			if(attribute!=null){
				setBorderWidthBottom(getSize(attribute));
			}
			attribute = attributes.get("border-width-left");
			if(attribute!=null){
				setBorderWidthLeft(getSize(attribute));
			}
			attribute = attributes.get("border-width-right");
			if(attribute!=null){
				setBorderWidthRight(getSize(attribute));
			}
			attribute = attributes.get("border-color");
			if(attribute!=null){
				setBorderColor(getColor(attribute));
			}
			attribute = attributes.get("border-color-left");
			if(attribute!=null){
				setBorderColorLeft(getColor(attribute));
			}
			attribute = attributes.get("border-color-right");
			if(attribute!=null){
				setBorderColorRight(getColor(attribute));
			}
			attribute = attributes.get("border-color-top");
			if(attribute!=null){
				setBorderColorTop(getColor(attribute));
			}
			attribute = attributes.get("border-color-bottom");
			if(attribute!=null){
				setBorderColorBottom(getColor(attribute));
			}
			attribute = attributes.get("padding");
			if(attribute!=null){
				setPadding(getSize(attribute));
			}
			attribute = attributes.get("padding-left");
			if(attribute!=null){
				setPaddingLeft(getSize(attribute));
			}
			attribute = attributes.get("padding-right");
			if(attribute!=null){
				setPaddingRight(getSize(attribute));
			}
			attribute = attributes.get("padding-top");
			if(attribute!=null){
				setPaddingTop(getSize(attribute));
			}
			attribute = attributes.get("padding-bottom");
			if(attribute!=null){
				setPaddingBottom(getSize(attribute));
			}
			attribute = attributes.get("rotation");
			if(attribute!=null){
				if(attribute.equals("left")){
					setRotation(90);
				}else if(attribute.equals("right")){
					setRotation(270);
				}else if(attribute.equals("up")){
					setRotation(0);
				}else if(attribute.equals("down")){
					setRotation(180);
				}
			}
			attribute = attributes.get("line-height");
			if(attribute!=null){
				setLeading(0,Float.parseFloat(attribute));
			}
			attribute = attributes.get("character-spacing");
			if(attribute!=null){
				fCharacterSpacing=getSize(attribute);
			}
		}

		void setContent(String content){
			content=content.replaceAll("\\\\n","\n");
			Chunk chunk = new Chunk(content,fFont);
			chunk.setCharacterSpacing(fCharacterSpacing);
			setPhrase(new Phrase(chunk));
		}
	}
}
